//   File: sparkfun_TB6612_two_motor_Adafruit_joystick_test.ino
//
//   Demonstrate use of the MME motor library for driving the Sparkfun TB6612
//   break-out board.  Run two motors at different directions and speeds
//   which are controlled by input from an Adafruit joystick board
//
//   Gerald Recktenwald, gerry@pdx.edu,  created 2017-11-05

#include <MMEmotor.h>

// -- Analog input pins for the joystick
const byte X_INPUT_PIN = 0;
const byte Y_INPUT_PIN = 1;
const int  ANALOG_MAX = 1023;    // Maximum value for analog input:  10-bit => 1023

// -- Interrupt pin and status for button clicks, Interrupt 0 is on pin 2 !!
const int  button_interrupt = 0;
const int  LED_pin = 12;
int  toggle_on = false;         //  Global variable stores state changed by button clicks

// Logic pins to control the motors: PWMA and PWMB must be pins capable of PWM output
// User needs to correctly wire these pins to corresponding contacts on TB6612 breakout
#define PWMA 10   // Motor A
#define AIN1 9
#define AIN2 8
#define STBY 7   // Standby pin of TB6612. Shared by both channels
#define PWMB 3   // Motor B
#define BIN1 4
#define BIN2 5

// -- Initialize MMEmotor objects that control motors on the paper roller and ink jet carriage
MMEmotor motorA = MMEmotor(AIN1, AIN2, PWMA, STBY);
MMEmotor motorB = MMEmotor(BIN1, BIN2, PWMB, STBY);

// ---------------------------------------------------------------------------
void setup() {
  Serial.begin(9600);

  // -- Register the interrupt handler
  attachInterrupt( button_interrupt, handle_click, RISING);  // Register interrupt handler

  // -- Configure LED_pin, which is used to indicate button state
  pinMode(LED_pin, OUTPUT);

  // -- Include joystick calibration code when a new joystick is used.
  //    The purpose is to find the analog input values when the joystick
  //    is in the neutral position.  This should be near the middle of the
  //    range, e.g. 511 for a 10-bit analog input.  However, in practice the
  //    "zero" of the joystick is different from the middle of the analog 
  //    input rage.  Run the calibration, to determine the analog input values,
  //    xzero and yzero when the joystick is in its neutral position.  Then
  //    set the XLOW, XHIGH values as xzero + 10 (say)
  //    and yzero - 10.  The offset of +/- 10 is arbitrary.  Use the same
  //    procedure to specify YLOW and YHIGH.  The
  //    threshold values define a deadband around the neutral setting of
  //    the potentiometer.  Any reading in the deadband is taken to be zero.
  //
#ifdef JOYSTICK_CALIBRATION

  int xpot, ypot;

  while ( 1 ) {
    xpot = analogRead(X_INPUT_PIN);
    ypot = analogRead(Y_INPUT_PIN);
    Serial.print(xpot);   Serial.print("   ");   Serial.println(ypot);
  }

#endif

}

// ---------------------------------------------------------------------------
void loop() {

  // -- Define thresholds for deadband around neutral position of the potentiometers
  const int XLOW = 492, XHIGH = 512;    
  const int YLOW = 504, YHIGH = 524;    
  
  int min_speed = 20, max_speed = 155, x_speed, y_speed;
 
  // -- Get info from joystick
  x_speed = get_speed(X_INPUT_PIN, XLOW, XHIGH, min_speed, max_speed, ANALOG_MAX);
  y_speed = get_speed(Y_INPUT_PIN, YLOW, YHIGH, min_speed, max_speed, ANALOG_MAX);

  // -- Spin both motors.  Direction determined by the sign of x_speed and y_speed
  motorA.moveAtSpeed(x_speed);
  motorB.moveAtSpeed(y_speed);

  // -- Turn LED on/off to indicate button state
  if ( toggle_on ) {
    digitalWrite(LED_pin, HIGH);
  } else {
    digitalWrite(LED_pin, LOW);
  }

  Serial.print(x_speed);   Serial.print("  ");   Serial.println(y_speed);
}

// --------------------------------------------------------------------
// Compute motor speed from potentiometer reading
int get_speed(int pot_pin, int zlow, int zhigh, int Smin, int Smax, int Amax) {

  float s;
  int pot_val, motor_speed;

  pot_val = analogRead(pot_pin);

  if (pot_val > zhigh) {
    s = Smin + (pot_val - zhigh) * float( Smax - Smin ) / float(Amax - zhigh);
  } else if (pot_val < zlow) {
    s = -Smax + pot_val * float( Smax - Smin ) / float(zlow) ;   // Amin = 0
  } else {
    s = 0;  // potentiometer reading in the deadband
  }
  
  return(int(s));
}

// --------------------------------------------------------------------
void  handle_click()
{
  static unsigned long last_interrupt_time = 0;       //  Zero only when code first runs
 
  unsigned long interrupt_time = millis();             //  Read the clock
  if ( interrupt_time - last_interrupt_time > 200 ) {  //  Only count clicks separated by 200 msec
    toggle_on = !toggle_on;
  }
  last_interrupt_time = interrupt_time;
}

